import cv2

def write(
  img,
  text,
  font=cv2.FONT_HERSHEY_SIMPLEX,
  pos=(0,0),
  font_scale=1,
  font_thickness=2,
  color=(255, 255, 255),
  bg_color=None,
  align="",
):
  text = str(text)
  x, y = pos
  (text_w, text_h), _ =  cv2.getTextSize(text, font, font_scale, font_thickness)
  if 'bottom' in align:
    y -= text_h
  if 'right' in align:
    x -= text_w

  if bg_color is not None:
    cv2.rectangle(
      img,
      (x, y),
      (x + text_w, y + text_h),
      bg_color,
      -1
    )
  cv2.putText(
    img,
    text,
    (x, int(y + text_h + font_scale - 1)),
    font,
    font_scale,
    color,
    font_thickness
  )

  return (text_w, text_h)


import cv2
import numpy as np


def many(images, text=None, border=False, text_color=(255,255,255)):
    w = int(np.ceil(len(images)**0.5))
    h = int(np.ceil(len(images)/w))
    if h < 2:
        return np.hstack(images)

    (ih, iw, ic) = images[0].shape
    result = np.zeros((h*ih, w*iw, ic), dtype=np.uint8)
    for index, image in enumerate(images):
        x0 = iw * (index % w)
        y0 = ih * int(np.floor(index / w))
        result[y0:y0+ih,x0:x0+iw,:] = image
    for index, image in enumerate(images):
        x0 = iw * (index % w)
        y0 = ih * int(np.floor(index / w))
        if text:
            cv2.putText(result, text[index], (x0+5, y0+20), cv2.FONT_HERSHEY_SIMPLEX, 0.5, text_color)
        if border:
            cv2.line(result, (x0+iw, y0), (x0+iw, y0+ih), border, 1)
            cv2.line(result, (x0, y0+ih), (x0+iw, y0+ih), border, 1)
    return result

def many_grays(images, **kwargs):
    converted = []
    for image in images:
        if len(image.shape) == 2:
            image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)
        converted.append(image)
    return many(converted, **kwargs)


import cv2
from collections import OrderedDict
import numpy as np


def filter(hsv, hue=[0,180], saturation=[0, 255], value=[0, 255]):
    lower = np.array([hue[0], saturation[0], value[0]])
    upper = np.array([hue[1], saturation[1], value[1]])
    mask = cv2.inRange(hsv, lower, upper)
    return cv2.bitwise_and(hsv, hsv, mask=mask)


def scan_hue(hsv, n_steps=12, **kwargs):
    s = 180 / n_steps
    ranges = [(int(i*s), int((i+1) * s)) for i in range(n_steps)]
    results = [filter(hsv, hue=r, **kwargs) for r in ranges]
    return results, ranges


def hist(hsv):
    h_bins = 50
    s_bins = 60
    histSize = [h_bins, s_bins]
    # hue varies from 0 to 179, saturation from 0 to 255
    h_ranges = [0, 180]
    s_ranges = [0, 256]
    ranges = h_ranges + s_ranges # concat lists
    # Use the 0-th and 1-st channels
    channels = [0, 1]
    return cv2.calcHist([hsv], channels, None, histSize, ranges, accumulate=False)


import cv2
import numpy as np


def autocrop_zeros(image):
    image = image.copy()
    while np.sum(image[-1]) == 0:
        # bottom
        image = image[:-1]
    while np.sum(image[0]) == 0:
        # top
        image = image[1:]
    while np.sum(image[:,-1]) == 0:
        # right
        image = image[:,:-1]
    while np.sum(image[:,0]) == 0:
        # left
        image = image[:,1:]
    return image


def scale(image, scale, interpolation=cv2.INTER_NEAREST):
    w = int(image.shape[1] * scale)
    h = int(image.shape[0] * scale)
    return cv2.resize(image, (w, h), interpolation=interpolation)


def crop(image, bounds):
    x, y, w, h = bounds

    # if x/y are negative adjust w/h and set x/y to zero
    if x < 0:
        w += x
        x = 0
    if y < 0:
        h += y
        y = 0

    return image[y:y+h,x:x+w]


def crop_ratio(image, bounds):
    ih, iw = image.shape[:2]
    x, y, w, h = bounds
    x = int(x * iw)
    w = int(w * iw)
    y = int(y * ih)
    h = int(h * ih)
    return crop(image, (x, y, w, h))


def threshold(image, value=127, max_value=255, type=cv2.THRESH_BINARY):
    gray = image
    if len(image.shape) == 3:
        gray = cv2.cvtColor(gray, cv2.COLOR_BGR2GRAY)
    _, mask = cv2.threshold(gray, value, max_value, type)
    return cv2.bitwise_and(image, image, mask=mask)



import cv2
import numpy as np


def polygon(image, points, fill=None, stroke=None, interiors=[], bg=None):
    pts = np.array(points, dtype=np.int32)
    pts = pts.reshape((-1,1,2))
    if fill is not None:
        cv2.fillPoly(image, [pts], color=fill)
    if stroke is not None:
        cv2.polylines(image, [pts], True, color=stroke)
    if bg is None:
        bg = tuple([0 for _ in fill])
    for shape in interiors:
        polygon(image, shape, fill=bg, stroke=stroke)


def paste(back, front, x1, y1):
    """
    Paste front onto back at x, y
    Optimized for non-alpha images
    """

    # if x or y is negative, cut front to fit
    if x1 < 0:
        front = front[:,abs(x1):]
        x1 = 0
    if y1 < 0:
        front = front[abs(y1):]
        y1 = 0

    bh, bw = back.shape[:2]
    fh, fw = front.shape[:2]
    x2 = x1+fw
    y2 = y1+fh

    # if x or y overflows right or bottom edges, cut front to fit
    if x2 > bw:
        dx = x2 - bw
        x2 = bw
        front = front[:,:-dx]
    if y2 > bh:
        dy = y2 - bh
        y2 = bh
        front = front[:-dy]

    back[y1:y2, x1:x2] = front


def paste_alpha(back, front, x, y):
    """
    Paste front onto back at x, y while preserving alpha channel
    Will throw error if target is out of bounds for back image
    """
    assert back.shape[2] == 4
    if front.shape[2] == 3:
        front = cv2.cvtColor(back, cv2.COLOR_BGR2BGRA)

    if x < 0:
        front = front[:, abs(x):]
        x = 0

    if y < 0:
        front = front[abs(y):]
        y = 0

    y_buffer = back.shape[0] - front.shape[0] - y
    if y_buffer < 0:
        front = front[:y_buffer]

    x_buffer = back.shape[1] - front.shape[1] - x
    if x_buffer < 0:
        front = front[:,:-x_buffer]

    # crop the overlay from both images
    bh, bw = back.shape[:2]
    fh, fw = front.shape[:2]
    x1, x2 = max(x, 0), min(x+fw, bw)
    y1, y2 = max(y, 0), min(y+fh, bh)
    back_cropped = back[y1:y2, x1:x2]

    alpha_front = front[:,:,3:4] / 255
    alpha_back = back_cropped[:,:,3:4] / 255

    # replace an area in back with overlay
    back[y1:y2, x1:x2, :3] = alpha_front * front[:,:,:3] + (1-alpha_front) * back_cropped[:,:,:3]
    back[y1:y2, x1:x2, 3:4] = (alpha_front + alpha_back) / (1 + alpha_front*alpha_back) * 255



import cv2
import numpy as np
from pathlib import Path

from . import draw
from . import filter
from . import hsv
from . import stack
from . import template
from . import text
from . import transform
from .input import wait_key, get_scaled_roi

__version__ = '0.0.5'

def count_colors(image):
    reshaped = image.reshape(-1, image.shape[-1])
    return np.unique(reshaped, axis=0, return_counts=True)

def top_color(image, exclude=[(0,0,0), (255,255,255)]):
    colors, counts = count_colors(image)
    for count, color in sorted(zip(counts, colors.tolist()))[::-1]:
        b, g, r = color[:3]
        if not (b, g, r) in exclude:
            return (b, g, r)

def force_alpha(image):
    if len(image.shape) == 2:
        return cv2.cvtColor(image, cv2.COLOR_GRAY2BGRA)
    if image.shape[2] == 3:
        return np.dstack([image, np.full(image.shape[:2], 255, dtype=np.uint8)])
    return image

def replace_color(image, color1, color2):
    r1, g1, b1 = color1[:3]
    r2, g2, b2 = color2[:3]

    red, green, blue = image[:,:,0], image[:,:,1], image[:,:,2]
    mask = (red == r1) & (green == g1) & (blue == b1)
    image[:,:,:3][mask] = [r2, g2, b2]


def remove_color_alpha(image, color1):
    r1, g1, b1 = color1

    red, green, blue = image[:,:,0], image[:,:,1], image[:,:,2]
    mask = (red == r1) & (green == g1) & (blue == b1)
    image[mask] = [0, 0, 0, 0]


#!/usr/bin/env python

import os
from pathlib import Path
import pkg_resources
from setuptools import setup, find_namespace_packages
import re

os.environ['PYTHON_EGG_CACHE'] = '.eggs'

with Path("urcv/__init__.py").open() as f:
    version = re.search('__version__\s*=\s*[\'"](.*)[\'"]\n', f.read()).group(1)

install_requires = [
    'imutils>=0.5.4',
    'opencv_python>=4.5.5.62',
    'pillow>=9.0.1',
    'scikit-learn>=1.0.2',
]

setup(
    name="urcv",
    version=version,
    packages=find_namespace_packages(),
    install_requires=install_requires,
    description="",
    author="",
    author_email="",
    url="https://github.com/chriscauley/urcv",
)



import cv2


def brightness(image, value=30):
    has_alpha = image.shape[-1] == 4
    if has_alpha:
        hsv = cv2.cvtColor(image, cv2.COLOR_BGRA2HSVA)
        h, s, v, a = cv2.split(hsv)
    else:
        hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        h, s, v = cv2.split(hsv)
    v = cv2.add(v,value)
    v[v > 255] = 255
    v[v < 0] = 0
    if has_alpha:
        final_hsv = cv2.merge((h, s, v, a))
        return cv2.cvtColor(final_hsv, cv2.COLOR_HSVA2BGRA)
    else:
        final_hsv = cv2.merge((h, s, v))
        return cv2.cvtColor(final_hsv, cv2.COLOR_HSV2BGR)


adjust_brightness = brightness

def chain(image, brightness=0, multiply=None):
    if brightness:
        image = adjust_brightness(image, brightness)
    if multiply is not None:
        image = cv2.multiply(image, multiply)
    return image
